#ifndef _CEBOARD_UCI_H
#define _CEBOARD_UCI_H


//_WIN32_WCE

#ifdef __cplusplus
extern "C" {
#endif


#define FILENAME_MAX    260
#define SIGINT			0
#define SIG_IGN			0
#if !defined(EINTR)
#define EINTR			0
#endif

extern int errno;


void system(char *);
void remove(char *);
char *getenv(char *);
void seedMT(int seed);
void signal(int a, int b);
int read(void *, char *buffer, int len);
time_t time(time_t *timer);
void MyExit(int);
char *strerror(int);

#ifndef NO_MIN_MAX
__forceinline int min(int a, int b) {
	if (a<b) {
		return a;
	} else {
		return b;
	}
}
__forceinline int max(int a, int b) {
	if (a>b) {
		return a;
	} else {
		return b;
	}
}
#endif

/*
#undef MallocInterleaved
#undef FreeInterleaved

void *MallocInterleaved(size_t cBytes, int cThreads);
void FreeInterleaved(void *pMemory, size_t cBytes);
*/

void EngineMove(char *sMove);
void EngineReady();
void EngineFeedback(int depth, int nodesSec, unsigned __int64 totalNodes, int nTime, char *sEval, char *sVariante);

#ifdef __cplusplus
} // extern "C"
#endif // } extern C


#define exit MyExit

#pragma warning(disable:4100 4127 4201 4214 4514)


#if defined(CEBOARD_UCI)
// C++ classes of the engine interface

// Classe d'accs  la registry

class CRegistryKey {
public:

	~CRegistryKey();
	CRegistryKey(HKEY hBranche,LPCTSTR sCle, BOOL bCreer = FALSE);

	operator HKEY() const {return m_hCle;};	// Rcupration du HANDLE

	BOOL LireValeurDefaut(LPTSTR sValeur, DWORD nSize);
	BOOL EcrireValeurDefaut(LPCTSTR sValeur);

	BOOL VerifierExistence(LPCTSTR sVal);

	int LireLongueurChaine(LPCTSTR sNom);
	BOOL LireChaine(LPCTSTR sNom, LPTSTR sValeur, DWORD nSize);
	BOOL EcrireChaine(LPCTSTR sNom,LPCTSTR sValeur);
	BOOL LireLong(LPCTSTR sNom, long *pnVal);
	BOOL EcrireLong(LPCTSTR sNom, long nVal);
	BOOL LireFloat(LPCTSTR sNom, float *pfVal);
	BOOL EcrireFloat(LPCTSTR sNom, float fVal);
	BOOL LireBlob(LPCTSTR sNom, LPVOID lpBlob, DWORD dwSize, DWORD *pdwRead);
	BOOL EcrireBlob(LPCTSTR sNom, LPVOID lpBlob, DWORD dwSize);

	BOOL EnumererChaine(DWORD &nIndice,
						LPTSTR sValeur, DWORD nValSize,
						LPTSTR pName, DWORD nNameSize);

	BOOL EnumererSousCles(DWORD &nIndice, LPTSTR pName, DWORD nNameSize);

	// Supprime une cl et toutes ses sous-cls
	static BOOL Delete(HKEY hBranche, LPCTSTR sCle);

	// Supprime une valeur
	BOOL SupprimerValeur(LPCTSTR sNom);

	BOOL Exists() const						{ return m_hCle != NULL; }
private:
	HKEY m_hCle;
};


/////////////////////////
// Circular buffer for pipe/stdin/stdout replacement
class CBuffer {
public:
	CBuffer(DWORD dwTaille);
	~CBuffer();
	DWORD GetDedans(void);
	void Dump(DWORD *pdwPosLecture, DWORD *pdwPosEcriture, char *sBuf);
	BOOL Ecrire(const char *sMsg);
	BOOL Lire(char *sMsg, DWORD *pdwTaille, BOOL bWait);
	void Gets(char *string, int n);
private:
	char *m_pBuf;
	DWORD m_dwTaille;
	DWORD m_PosLecture;
	DWORD m_PosEcriture;
	CRITICAL_SECTION m_cs;
	HANDLE m_hEventWrite;
	HANDLE m_hEventRead;
};

#define GAME_LEN 8192

class CUCIEngine : public IEngine {
public:
	CUCIEngine(ICEBoard20 *pCEBoard);
	virtual ~CUCIEngine();

	// IEngine implementation
	BOOL GetProperties(EngineProperties *pProps);
	void StartGame(BOOL bPlayBlack, BOOL bAnalyze, LPCTSTR sFen);
	void EndGame();
	void Release();
	void SendMove(LPCTSTR sMove);
	BOOL ShowOptions(HWND hWnd);
	HMENU GetMenu();
	BOOL HandleCommand(HWND hWnd, WPARAM wParam, LPARAM lParam);
	void SetPosition(LPCTSTR sFEN);
	void MoveNow();
	BOOL IsAlive();
	void GetMessages(BOOL bWantMessages);

	// Internal methods
	void EngineReply(char *msg);
	void FatalError(const char *msg);
	void GameFinished(const wchar_t *wsResult, const wchar_t *wsComment);
	int IsGameFinished();

protected:

	// Data members
	HANDLE m_hEngineThread;
	ICEBoard20 *m_pCEBoard;
	// Is the game finished
	BOOL m_bFinished;
	// Is the engine in analysis mode ?
	BOOL m_bAnalyze;
	// Is engine thinking?
	BOOL m_bThinking;
	// Does host want messages
	BOOL m_bWantMessages;
	// Position command
	BOOL m_bAddMoves;
	char m_sGame[GAME_LEN];
	int m_nGamePointer;
	// For synchronous replys
	HANDLE m_hReplyReceived;
	int m_nReplyLength;
	char m_sReply[32];
	// Feedback "cache"
	int m_nDepth;


	HMENU m_hMenu;

	// Time options
	long m_nTimeControlType;
	long m_nGameTime;
	long m_nMoveCount;
	BOOL m_bMoveIncrement;
	long m_nIncrementValue;
	long m_nFixedTime;

	// Options (engine specific)
	long m_nHash;
	TCHAR m_sBookPath[MAX_PATH];

	// Implementation
	void WaitFor(LPCSTR sCommand, LPCSTR sReply);
	void CheckShowAnalysis();

	void SetThinking(BOOL bThinking);

	void SendTimeCommands(BOOL bForce, long nTimeControlType, long nGameTime, long nMoveCount, BOOL bMoveIncrement, long nIncrementValue, long nFixedTime);
	void SendOtherOptionsCommands(BOOL bForce, int nHash, LPCTSTR sBook, BOOL bLog);

	static BOOL CALLBACK GlobalTimeControlProc(HWND hDlg, UINT nMsg, WPARAM wParam, LPARAM lParam);
	BOOL TimeControlProc(HWND hDlg, UINT nMsg, WPARAM wParam, LPARAM lParam);
	void SetTimeValues(HWND hDlg);
	BOOL ReadTimeValues(HWND hDlg, long *pnTimeControlType, long *pnGameTime, long *pnMoveCount, BOOL *pbMoveIncrement, long *pnIncrementValue, long *pnFixedTime);

	static BOOL CALLBACK GlobalOtherOptionsProc(HWND hDlg, UINT nMsg, WPARAM wParam, LPARAM lParam);
	BOOL OtherOptionsProc(HWND hDlg, UINT nMsg, WPARAM wParam, LPARAM lParam);
	void SetOtherOptionsValues(HWND hDlg);
	BOOL ReadOtherOptionsValues(HWND hDlg, long *pnHash, TCHAR *sBuf, DWORD dwLen, BOOL *pbLog);


	void AddMove(LPCTSTR sMove);
};

#endif // CEBOARD_UCI

#endif
